import React from "react";
import { Props, NodeType, MathboxSelection } from "mathbox";
import { WithChildren } from "./types";
import { ParentNodeTypes } from "./util";
type LiveProps<P> = {
    [K in keyof P]: (t: number, dt: number) => P[K];
};
type MathboxComponent<T extends NodeType> = React.ForwardRefExoticComponent<(T extends ParentNodeTypes ? WithChildren<Props[T]> : Props[T]) & React.RefAttributes<MathboxSelection<T>> & {
    liveProps?: LiveProps<Props[T]>;
}>;
/**
 * Component wrapper for mathbox [`area`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#dataarea).
 */
export declare const Area: MathboxComponent<"area">;
/**
 * Component wrapper for mathbox [`array`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#dataarray).
 */
declare const MBArray: MathboxComponent<"array">;
export { MBArray as Array };
/**
 * Component wrapper for mathbox [`axis`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawaxis).
 */
export declare const Axis: MathboxComponent<"axis">;
/**
 * Component wrapper for mathbox [`camera`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#cameracamera).
 */
export declare const Camera: MathboxComponent<"camera">;
/**
 * Component wrapper for mathbox [`cartesian`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewcartesian).
 */
export declare const Cartesian: MathboxComponent<"cartesian">;
/**
 * Component wrapper for mathbox [`cartesian4`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewcartesian4).
 */
export declare const Cartesian4: MathboxComponent<"cartesian4">;
/**
 * Component wrapper for mathbox [`clamp`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorclamp).
 */
export declare const Clamp: MathboxComponent<"clamp">;
/**
 * Component wrapper for mathbox [`clock`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#timeclock).
 */
export declare const Clock: MathboxComponent<"clock">;
/**
 * Component wrapper for mathbox [`compose`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#rttcompose).
 */
export declare const Compose: MathboxComponent<"compose">;
/**
 * Component wrapper for mathbox [`dom`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#overlaydom).
 */
export declare const Dom: MathboxComponent<"dom">;
/**
 * Component wrapper for mathbox [`face`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawface).
 */
export declare const Face: MathboxComponent<"face">;
/**
 * Component wrapper for mathbox [`format`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#textformat).
 */
export declare const Format: MathboxComponent<"format">;
/**
 * Component wrapper for mathbox [`fragment`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformfragment).
 */
export declare const Fragment: MathboxComponent<"fragment">;
/**
 * Component wrapper for mathbox [`grid`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawgrid).
 */
export declare const Grid: MathboxComponent<"grid">;
/**
 * Component wrapper for mathbox [`group`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#basegroup).
 */
export declare const Group: MathboxComponent<"group">;
/**
 * Component wrapper for mathbox [`grow`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorgrow).
 */
export declare const Grow: MathboxComponent<"grow">;
/**
 * Component wrapper for mathbox [`html`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#overlayhtml).
 */
export declare const Html: MathboxComponent<"html">;
/**
 * Component wrapper for mathbox [`inherit`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#baseinherit).
 */
export declare const Inherit: MathboxComponent<"inherit">;
/**
 * Component wrapper for mathbox [`interval`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datainterval).
 */
export declare const Interval: MathboxComponent<"interval">;
/**
 * Component wrapper for mathbox [`join`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorjoin).
 */
export declare const Join: MathboxComponent<"join">;
/**
 * Component wrapper for mathbox [`label`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#textlabel).
 */
export declare const Label: MathboxComponent<"label">;
/**
 * Component wrapper for mathbox [`layer`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformlayer).
 */
export declare const Layer: MathboxComponent<"layer">;
/**
 * Component wrapper for mathbox [`lerp`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorlerp).
 */
export declare const Lerp: MathboxComponent<"lerp">;
/**
 * Component wrapper for mathbox [`line`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawline).
 */
export declare const Line: MathboxComponent<"line">;
/**
 * Component wrapper for mathbox [`mask`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformmask).
 */
export declare const Mask: MathboxComponent<"mask">;
/**
 * Component wrapper for mathbox [`matrix`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datamatrix).
 */
export declare const Matrix: MathboxComponent<"matrix">;
/**
 * Component wrapper for mathbox [`memo`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatormemo).
 */
export declare const Memo: MathboxComponent<"memo">;
/**
 * Component wrapper for mathbox [`move`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentmove).
 */
export declare const Move: MathboxComponent<"move">;
/**
 * Component wrapper for mathbox [`now`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#timenow).
 */
export declare const Now: MathboxComponent<"now">;
/**
 * Component wrapper for mathbox [`play`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentplay).
 */
export declare const Play: MathboxComponent<"play">;
/**
 * Component wrapper for mathbox [`point`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawpoint).
 */
export declare const Point: MathboxComponent<"point">;
/**
 * Component wrapper for mathbox [`polar`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewpolar).
 */
export declare const Polar: MathboxComponent<"polar">;
/**
 * Component wrapper for mathbox [`present`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentpresent).
 */
export declare const Present: MathboxComponent<"present">;
/**
 * Component wrapper for mathbox [`readback`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorreadback).
 */
export declare const Readback: MathboxComponent<"readback">;
/**
 * Component wrapper for mathbox [`repeat`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorrepeat).
 */
export declare const Repeat: MathboxComponent<"repeat">;
/**
 * Component wrapper for mathbox [`resample`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorresample).
 */
export declare const Resample: MathboxComponent<"resample">;
/**
 * Component wrapper for mathbox [`retext`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#textretext).
 */
export declare const Retext: MathboxComponent<"retext">;
/**
 * Component wrapper for mathbox [`reveal`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentreveal).
 */
export declare const Reveal: MathboxComponent<"reveal">;
/**
 * Component wrapper for mathbox [`rtt`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#rttrtt).
 */
export declare const Rtt: MathboxComponent<"rtt">;
/**
 * Component wrapper for mathbox [`scale`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datascale).
 */
export declare const Scale: MathboxComponent<"scale">;
/**
 * Component wrapper for mathbox [`shader`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#shadershader).
 */
export declare const Shader: MathboxComponent<"shader">;
/**
 * Component wrapper for mathbox [`slice`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorslice).
 */
export declare const Slice: MathboxComponent<"slice">;
/**
 * Component wrapper for mathbox [`slide`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentslide).
 */
export declare const Slide: MathboxComponent<"slide">;
/**
 * Component wrapper for mathbox [`spherical`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewspherical).
 */
export declare const Spherical: MathboxComponent<"spherical">;
/**
 * Component wrapper for mathbox [`split`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorsplit).
 */
export declare const Split: MathboxComponent<"split">;
/**
 * Component wrapper for mathbox [`spread`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorspread).
 */
export declare const Spread: MathboxComponent<"spread">;
/**
 * Component wrapper for mathbox [`step`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentstep).
 */
export declare const Step: MathboxComponent<"step">;
/**
 * Component wrapper for mathbox [`stereographic`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewstereographic).
 */
export declare const Stereographic: MathboxComponent<"stereographic">;
/**
 * Component wrapper for mathbox [`stereographic4`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewstereographic4).
 */
export declare const Stereographic4: MathboxComponent<"stereographic4">;
/**
 * Component wrapper for mathbox [`strip`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawstrip).
 */
export declare const Strip: MathboxComponent<"strip">;
/**
 * Component wrapper for mathbox [`subdivide`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorsubdivide).
 */
export declare const Subdivide: MathboxComponent<"subdivide">;
/**
 * Component wrapper for mathbox [`surface`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawsurface).
 */
export declare const Surface: MathboxComponent<"surface">;
/**
 * Component wrapper for mathbox [`swizzle`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorswizzle).
 */
export declare const Swizzle: MathboxComponent<"swizzle">;
/**
 * Component wrapper for mathbox [`text`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#texttext).
 */
export declare const Text: MathboxComponent<"text">;
/**
 * Component wrapper for mathbox [`ticks`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawticks).
 */
export declare const Ticks: MathboxComponent<"ticks">;
/**
 * Component wrapper for mathbox [`transform`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformtransform).
 */
export declare const Transform: MathboxComponent<"transform">;
/**
 * Component wrapper for mathbox [`transform4`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformtransform4).
 */
export declare const Transform4: MathboxComponent<"transform4">;
/**
 * Component wrapper for mathbox [`transpose`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatortranspose).
 */
export declare const Transpose: MathboxComponent<"transpose">;
/**
 * Component wrapper for mathbox [`unit`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#baseunit).
 */
export declare const Unit: MathboxComponent<"unit">;
/**
 * Component wrapper for mathbox [`vector`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawvector).
 */
export declare const Vector: MathboxComponent<"vector">;
/**
 * Component wrapper for mathbox [`vertex`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformvertex).
 */
export declare const Vertex: MathboxComponent<"vertex">;
/**
 * Component wrapper for mathbox [`view`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewview).
 */
export declare const View: MathboxComponent<"view">;
/**
 * Component wrapper for mathbox [`volume`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datavolume).
 */
export declare const Volume: MathboxComponent<"volume">;
/**
 * Component wrapper for mathbox [`voxel`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datavoxel).
 */
export declare const Voxel: MathboxComponent<"voxel">;
