var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { forwardRef, useRef, useContext, useEffect, useReducer, useImperativeHandle, } from "react";
import MathboxAPIContext from "./MathboxAPIContext";
import { isRootDestroyed, isSelectionParent, canNodeHaveChildren, capitalize, } from "./util";
var mathboxComponentFactory = function (type) {
    var canHaveChildren = canNodeHaveChildren(type);
    var componentName = capitalize(type);
    var Comp = function (props, ref) {
        var _a = useReducer(function (x) { return x + 1; }, 0), _ignored = _a[0], forceUpdate = _a[1];
        var parent = useContext(MathboxAPIContext);
        var selection = useRef(null);
        var prevLiveProps = useRef(undefined);
        useEffect(function () { return function () {
            if (selection.current) {
                selection.current.remove();
                selection.current = null;
            }
        }; }, []);
        useImperativeHandle(ref, function () { return selection.current; });
        var children = props.children, liveProps = props.liveProps, others = __rest(props, ["children", "liveProps"]);
        useEffect(function () {
            if (!parent)
                return;
            if (isRootDestroyed(parent)) {
                forceUpdate();
                return;
            }
            if (selection.current) {
                if (!isSelectionParent(selection.current, parent)) {
                    selection.current.remove();
                    selection.current = null;
                    forceUpdate();
                }
            }
            if (!selection.current) {
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-expect-error
                selection.current = parent[type](others, liveProps);
                forceUpdate();
            }
            else {
                /**
                 * If liveProps have changed, remove all the old liveProps.
                 * (The same prop cannot be re-assigned live without being un-assigned
                 * first.)
                 *
                 * We could unbind just the ones that have changed, but simpler to
                 * unbind them all.
                 */
                if (prevLiveProps.current && liveProps !== prevLiveProps.current) {
                    Object.keys(prevLiveProps.current).forEach(function (key) {
                        // @ts-expect-error this is not in ts yet
                        selection.current.unbind(key);
                    });
                }
                if (liveProps && liveProps !== prevLiveProps.current) {
                    selection.current.bind(liveProps);
                }
                selection.current.set(others);
            }
            prevLiveProps.current = liveProps;
        }, [parent, others, liveProps]);
        if (!canHaveChildren) {
            if (props.children) {
                throw new Error("Component <".concat(componentName, " /> cannot have children."));
            }
            return null;
        }
        return (React.createElement(MathboxAPIContext.Provider, { value: selection.current }, props.children));
    };
    /**
     * The line below gives a TS error without explicit any.
     * But if you replace each generic T above with a specific instance, e.g.,
     * 'cartesian', there is no error.
     *
     * So an alternative without "any" would be be to copy the definition of Comp
     * with all ~64 different node types.
     *
     * I'm not 100% sure why there's a ts error here, but the above indicates to
     * me that this is actually type-safe. And I do not want that much copy pasta.
     * So instead, use the any and put explicit return type on the factory.
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    return forwardRef(Comp);
};
/**
 * Component wrapper for mathbox [`area`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#dataarea).
 */
export var Area = mathboxComponentFactory("area");
/**
 * Component wrapper for mathbox [`array`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#dataarray).
 */
var MBArray = mathboxComponentFactory("array");
export { MBArray as Array };
/**
 * Component wrapper for mathbox [`axis`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawaxis).
 */
export var Axis = mathboxComponentFactory("axis");
/**
 * Component wrapper for mathbox [`camera`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#cameracamera).
 */
export var Camera = mathboxComponentFactory("camera");
/**
 * Component wrapper for mathbox [`cartesian`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewcartesian).
 */
export var Cartesian = mathboxComponentFactory("cartesian");
/**
 * Component wrapper for mathbox [`cartesian4`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewcartesian4).
 */
export var Cartesian4 = mathboxComponentFactory("cartesian4");
/**
 * Component wrapper for mathbox [`clamp`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorclamp).
 */
export var Clamp = mathboxComponentFactory("clamp");
/**
 * Component wrapper for mathbox [`clock`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#timeclock).
 */
export var Clock = mathboxComponentFactory("clock");
/**
 * Component wrapper for mathbox [`compose`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#rttcompose).
 */
export var Compose = mathboxComponentFactory("compose");
/**
 * Component wrapper for mathbox [`dom`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#overlaydom).
 */
export var Dom = mathboxComponentFactory("dom");
/**
 * Component wrapper for mathbox [`face`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawface).
 */
export var Face = mathboxComponentFactory("face");
/**
 * Component wrapper for mathbox [`format`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#textformat).
 */
export var Format = mathboxComponentFactory("format");
/**
 * Component wrapper for mathbox [`fragment`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformfragment).
 */
export var Fragment = mathboxComponentFactory("fragment");
/**
 * Component wrapper for mathbox [`grid`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawgrid).
 */
export var Grid = mathboxComponentFactory("grid");
/**
 * Component wrapper for mathbox [`group`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#basegroup).
 */
export var Group = mathboxComponentFactory("group");
/**
 * Component wrapper for mathbox [`grow`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorgrow).
 */
export var Grow = mathboxComponentFactory("grow");
/**
 * Component wrapper for mathbox [`html`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#overlayhtml).
 */
export var Html = mathboxComponentFactory("html");
/**
 * Component wrapper for mathbox [`inherit`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#baseinherit).
 */
export var Inherit = mathboxComponentFactory("inherit");
/**
 * Component wrapper for mathbox [`interval`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datainterval).
 */
export var Interval = mathboxComponentFactory("interval");
/**
 * Component wrapper for mathbox [`join`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorjoin).
 */
export var Join = mathboxComponentFactory("join");
/**
 * Component wrapper for mathbox [`label`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#textlabel).
 */
export var Label = mathboxComponentFactory("label");
/**
 * Component wrapper for mathbox [`layer`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformlayer).
 */
export var Layer = mathboxComponentFactory("layer");
/**
 * Component wrapper for mathbox [`lerp`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorlerp).
 */
export var Lerp = mathboxComponentFactory("lerp");
/**
 * Component wrapper for mathbox [`line`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawline).
 */
export var Line = mathboxComponentFactory("line");
/**
 * Component wrapper for mathbox [`mask`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformmask).
 */
export var Mask = mathboxComponentFactory("mask");
/**
 * Component wrapper for mathbox [`matrix`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datamatrix).
 */
export var Matrix = mathboxComponentFactory("matrix");
/**
 * Component wrapper for mathbox [`memo`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatormemo).
 */
export var Memo = mathboxComponentFactory("memo");
/**
 * Component wrapper for mathbox [`move`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentmove).
 */
export var Move = mathboxComponentFactory("move");
/**
 * Component wrapper for mathbox [`now`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#timenow).
 */
export var Now = mathboxComponentFactory("now");
/**
 * Component wrapper for mathbox [`play`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentplay).
 */
export var Play = mathboxComponentFactory("play");
/**
 * Component wrapper for mathbox [`point`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawpoint).
 */
export var Point = mathboxComponentFactory("point");
/**
 * Component wrapper for mathbox [`polar`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewpolar).
 */
export var Polar = mathboxComponentFactory("polar");
/**
 * Component wrapper for mathbox [`present`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentpresent).
 */
export var Present = mathboxComponentFactory("present");
/**
 * Component wrapper for mathbox [`readback`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorreadback).
 */
export var Readback = mathboxComponentFactory("readback");
/**
 * Component wrapper for mathbox [`repeat`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorrepeat).
 */
export var Repeat = mathboxComponentFactory("repeat");
/**
 * Component wrapper for mathbox [`resample`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorresample).
 */
export var Resample = mathboxComponentFactory("resample");
/**
 * Component wrapper for mathbox [`retext`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#textretext).
 */
export var Retext = mathboxComponentFactory("retext");
/**
 * Component wrapper for mathbox [`reveal`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentreveal).
 */
export var Reveal = mathboxComponentFactory("reveal");
/**
 * Component wrapper for mathbox [`rtt`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#rttrtt).
 */
export var Rtt = mathboxComponentFactory("rtt");
/**
 * Component wrapper for mathbox [`scale`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datascale).
 */
export var Scale = mathboxComponentFactory("scale");
/**
 * Component wrapper for mathbox [`shader`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#shadershader).
 */
export var Shader = mathboxComponentFactory("shader");
/**
 * Component wrapper for mathbox [`slice`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorslice).
 */
export var Slice = mathboxComponentFactory("slice");
/**
 * Component wrapper for mathbox [`slide`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentslide).
 */
export var Slide = mathboxComponentFactory("slide");
/**
 * Component wrapper for mathbox [`spherical`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewspherical).
 */
export var Spherical = mathboxComponentFactory("spherical");
/**
 * Component wrapper for mathbox [`split`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorsplit).
 */
export var Split = mathboxComponentFactory("split");
/**
 * Component wrapper for mathbox [`spread`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorspread).
 */
export var Spread = mathboxComponentFactory("spread");
/**
 * Component wrapper for mathbox [`step`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#presentstep).
 */
export var Step = mathboxComponentFactory("step");
/**
 * Component wrapper for mathbox [`stereographic`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewstereographic).
 */
export var Stereographic = mathboxComponentFactory("stereographic");
/**
 * Component wrapper for mathbox [`stereographic4`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewstereographic4).
 */
export var Stereographic4 = mathboxComponentFactory("stereographic4");
/**
 * Component wrapper for mathbox [`strip`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawstrip).
 */
export var Strip = mathboxComponentFactory("strip");
/**
 * Component wrapper for mathbox [`subdivide`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorsubdivide).
 */
export var Subdivide = mathboxComponentFactory("subdivide");
/**
 * Component wrapper for mathbox [`surface`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawsurface).
 */
export var Surface = mathboxComponentFactory("surface");
/**
 * Component wrapper for mathbox [`swizzle`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatorswizzle).
 */
export var Swizzle = mathboxComponentFactory("swizzle");
/**
 * Component wrapper for mathbox [`text`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#texttext).
 */
export var Text = mathboxComponentFactory("text");
/**
 * Component wrapper for mathbox [`ticks`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawticks).
 */
export var Ticks = mathboxComponentFactory("ticks");
/**
 * Component wrapper for mathbox [`transform`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformtransform).
 */
export var Transform = mathboxComponentFactory("transform");
/**
 * Component wrapper for mathbox [`transform4`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformtransform4).
 */
export var Transform4 = mathboxComponentFactory("transform4");
/**
 * Component wrapper for mathbox [`transpose`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#operatortranspose).
 */
export var Transpose = mathboxComponentFactory("transpose");
/**
 * Component wrapper for mathbox [`unit`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#baseunit).
 */
export var Unit = mathboxComponentFactory("unit");
/**
 * Component wrapper for mathbox [`vector`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#drawvector).
 */
export var Vector = mathboxComponentFactory("vector");
/**
 * Component wrapper for mathbox [`vertex`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#transformvertex).
 */
export var Vertex = mathboxComponentFactory("vertex");
/**
 * Component wrapper for mathbox [`view`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#viewview).
 */
export var View = mathboxComponentFactory("view");
/**
 * Component wrapper for mathbox [`volume`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datavolume).
 */
export var Volume = mathboxComponentFactory("volume");
/**
 * Component wrapper for mathbox [`voxel`](https://gitgud.io/unconed/mathbox/-/blob/master/docs/primitives.md#datavoxel).
 */
export var Voxel = mathboxComponentFactory("voxel");
//# sourceMappingURL=components.js.map